<?php

/**
 * @file
 */

/**
 * GTFS schedule page.
 */
function gtfs_schedule_page($route) {

  $agency = variable_get('gtfs_schedule_agency', '');
  $date_format = variable_get('gtfs_schedule_timeformat', GTFS_SCHEDULE_TIMEFORMAT);

  // Get path for building links.
  $present_path = $_GET['q'];

  if (variable_get('gtfs_schedule_route_name_type', GTFS_SCHEDULE_DEFAULT_ROUTE_NAME_TYPE) !== 'route_id') {
    $route_id = gtfs_schedule_get_route_id($route);
  }
  else {
    $route_id = $route;
  }

  if (!$route_id) {
    return gtfs_schedule_error();
  }

  // Configure the time format.
  $arguments['timeFormat'] = $date_format;

  // Limit stops to timepoints.
  if (variable_get('gtfs_schedule_timepoints_only_default', FALSE) && empty($arguments['timepointsOnly'])) {
    $arguments['filter'] = ['timepoint' => ['=' => '1']];
  }

  // Allow other modules to alter the query arguments.
  drupal_alter('gtfs_schedule_request_arguments', $arguments);

  // Generate the full URL request.
  $service_endpoint = "agencies/{$agency}/routes/{$route_id}/schedules";

  $schedule = gtfs_schedule_request($service_endpoint, $arguments, TRUE);

  if (empty($schedule->data)) {
    return gtfs_schedule_error();
  }

  if (!isset($_GET['direction_id']) || !isset($_GET['service_id'])) {

    // Set page title.
    $title = "{$schedule->meta->route->route_short_name} {$schedule->meta->route->route_long_name}" . ' ' . t('Schedules');

    drupal_alter('gtfs_schedule_title', $title, $schedule);
    drupal_set_title($title);

    // Return links.
    $output['gtfs_schedule_header'] = [
      '#theme' => 'gtfs_schedule_table',
      '#schedules' => $schedule,
      '#route_url_id' => $route,
    ];

    return $output;
  }

  // Build the header from the stops.
  $header = [];

  foreach ($schedule->data->stops as $stop) {
    $header[] = [
      'data' => $stop->stop_name,
      'scope' => 'col',
    ];
  }

  // Alter the table header.
  drupal_alter('gtfs_schedule_header', $header, $schedule);

  // Reformatting data array to add trip_id and max time information.
  $rows = [];
  $last = 0;
  $next_day = 0;
  foreach ($schedule->data->trips as $row) {

    if (!count($row->stop_times)) {
      continue;
    }

    // Cycle across the stop times.
    $data = array_map(function ($stop) {
      return isset($stop->arrival_time) ? $stop->arrival_time : '';
    }, $row->stop_times);

    $filtered = array_filter($data);
    $max_date = date_create_from_format($date_format, end($filtered));
    $timestamp = $max_date->getTimestamp();

    // Check to see if we've moved into the next day.
    if ($timestamp < $last) {
      $next_day = 60 * 60 * 24;
    }

    // Add adjustment for next day.
    $timestamp += $next_day;

    // Save last for next loop.
    $last = $timestamp;

    // Set empty fields to placeholder.
    foreach ($data as $key => $value) {
      if ($value == '') {
        $data[$key] = variable_get('gtfs_schedule_empty_placeholder', GTFS_SCHEDULE_EMPTY_PLACEHOLDER);
      }
    }

    $rows[] = [
      'id' => $row->trip_id,
      'data-max-time' => $timestamp,
      'data' => $data,
    ];
  }

  // Could be used to alter the page title.
  drupal_alter('gtfs_schedule_data', $schedule);

  // Begin output.
  $output = [];

  // Set page title.
  $route_name = "{$schedule->meta->route->route_short_name} {$schedule->meta->route->route_long_name}";

  $direction_names = gtfs_schedule_get_direction_id_names($schedule);

  $direction = $direction_names[$schedule->meta->direction_id];
  $service_name = isset($schedule->data->calendar_date)
    ? $schedule->data->calendar_date->name
    : isset($schedule->data->service->name)
      ? $schedule->data->service->name
      : '';

  $title = "{$route_name} {$direction} {$service_name}";
  drupal_alter('gtfs_schedule_title', $title, $schedule);
  drupal_set_title($title);

  // Attach header.
  $output['gtfs_schedule_header'] = [
    '#theme' => 'gtfs_schedule_header',
    '#schedule' => $schedule,
    '#route_url_id' => $route,
  ];

  if (count($rows)) {

    if (isset($schedule->data->calendar_date)) {
      $service_name = $schedule->data->calendar_date->name;
    }

    $caption = "{$route_name} - {$direction} - {$service_name}";
    drupal_alter('gtfs_schedule_caption', $caption, $schedule);

    $output['GTFS_schedule_table'] = [
      '#theme' => 'table',
      '#attributes' => [
        'id' => 'route_schedule',
        'data-tablesaw-mode' => 'swipe',
        'data-tablesaw-minimap' => NULL,
        'class' => 'tablesaw tablesaw-swipe',
      ],
      '#header' => $header,
      '#rows' => $rows,
      '#caption' => $caption,
    ];
  }
  else {
    $output['GTFS_schedule_no_trips'] = [
      '#markup' => t('There are no trips for this route on this date in this direction.'),
    ];
  }

  // Could be used to wrap table with helpful links.
  drupal_alter('gtfs_schedule_output', $output);

  return $output;
}

/**
 * Returns error message markup.
 */
function gtfs_schedule_error() {
  $output = [
    'error' =>
      [
        '#markup' => variable_get('gtfs_schedule_no_schedule_message', GTFS_SCHEDULE_NOSCHEDULE_MESSAGE),
      ],
  ];

  drupal_alter('gtfs_schedule_error_message', $output);

  return $output;
}

/**
 * Converts short and long route codes to route id.
 */
function gtfs_schedule_get_route_id($query) {

  $agency = variable_get('gtfs_schedule_agency', '');
  $service_endpoint = "agencies/{$agency}/routes";

  $routes = gtfs_schedule_request($service_endpoint);

  if (empty($routes->data)) {
    return FALSE;
  }

  $key = variable_get('gtfs_schedule_route_name_type', GTFS_SCHEDULE_DEFAULT_ROUTE_NAME_TYPE);

  // Case insensitive array search across a particular field.
  $index = array_search(strtolower($query), array_map('strtolower', array_column($routes->data, $key)));

  if ($index !== FALSE && !empty($routes->data[$index]->route_id)) {
    return $routes->data[$index]->route_id;
  }

  return FALSE;
}

/**
 * GTFS schedule settings form.
 */
function gtfs_schedule_settings_form() {

  $form['gtfs_schedule_gtfs_server_url'] = [
    '#type' => 'textfield',
    '#title' => t('GTFS server URL'),
    '#default_value' => variable_get('gtfs_schedule_gtfs_server_url', ''),
    '#description' => t('A Drupal GTFS server URL that has parsed a GTFS feed.'),
  ];

  if(variable_get('gtfs_schedule_gtfs_server_url', '')) {
    $url = variable_get('gtfs_schedule_gtfs_server_url', '') . '/gtfs/api/v1';

    if(variable_get('gtfs_schedule_feed_id', GTFS_SCHEDULE_FEED_ID)) {
      $url .= '?feed_id=' . variable_get('gtfs_schedule_feed_id', GTFS_SCHEDULE_FEED_ID);
    }

    $form['gtfs_schedule_gtfs_server_url']['#description'] .=
    '<br />' . t('Current URL endpoint: !url', ['!url' => l($url, $url, ['absolute' => TRUE, 'attributes' => ['target' => '_blank']])]);
  }

  $form['gtfs_schedule_agency'] = [
    '#type' => 'textfield',
    '#title' => t('Agency'),
    '#default_value' => variable_get('gtfs_schedule_agency', ''),
    '#description' => t('The agency name of the GTFS feed imported into the Drupal GTFS server.'),
  ];

  $form['gtfs_schedule_route_name_type'] = [
    '#type' => 'select',
    '#title' => t('Route Name Type'),
    '#default_value' => variable_get('gtfs_schedule_route_name_type', GTFS_SCHEDULE_DEFAULT_ROUTE_NAME_TYPE),
    '#options' => [
      'route_id' => t('Route ID'),
      'route_short_name' => t('Route short name'),
      'route_long_name' => t('Route long name'),
    ],
    '#description' => t('Determines what route name type will be used in the URL.'),
  ];

  $form['gtfs_schedule_base_url'] = [
    '#type' => 'textfield',
    '#title' => t('Base URL for schedule pages'),
    '#default_value' => variable_get('gtfs_schedule_base_url', GTFS_SCHEDULE_BASE_URL),
    '#description' => t('The base URL you would like to see the schedules listed.'),
  ];

  $form['gtfs_schedule_timeformat'] = [
    '#type' => 'textfield',
    '#title' => t('Time format'),
    '#default_value' => variable_get('gtfs_schedule_timeformat', GTFS_SCHEDULE_TIMEFORMAT),
    '#description' => t('Schedule time format for stoptimes. Current format example: %example', ['%example' => date(variable_get('gtfs_schedule_timeformat', GTFS_SCHEDULE_TIMEFORMAT))]),
  ];

  $form['gtfs_schedule_empty_placeholder'] = [
    '#type' => 'textfield',
    '#title' => t('Empty value placeholder'),
    '#default_value' => variable_get('gtfs_schedule_empty_placeholder', GTFS_SCHEDULE_EMPTY_PLACEHOLDER),
    '#description' => t('The text shown when there is no stoptime for a given trip x stop combination.'),
  ];

  $form['gtfs_schedule_timepoints_only_default'] = [
    '#type' => 'checkbox',
    '#title' => t('Default to show only stops that are flagged as timepoints'),
    '#default_value' => variable_get('gtfs_schedule_timepoints_only_default', FALSE),
    '#description' => t('Check this box to limit a schedule to stops that have defined timepoints. This can be overridden using timepointsOnly=false parameter.'),
  ];

  $form['gtfs_schedule_no_schedule_message'] = [
    '#type' => 'textarea',
    '#title' => t('No Schedule Message'),
    '#default_value' => variable_get('gtfs_schedule_no_schedule_message', GTFS_SCHEDULE_NOSCHEDULE_MESSAGE),
    '#description' => t('Error message returned when schedule data is not available.'),
  ];

  $form['gtfs_schedule_request_cache_age_limit'] = [
    '#type' => 'select',
    '#title' => t('Request cache age limit'),
    '#default_value' => variable_get('gtfs_schedule_request_cache_age_limit', 0),
    '#options' => [
      '0' => t('Do not use cache'),
      '60' => t('1 minute'),
      '300' => t('5 minutes'),
      '600' => t('10 minutes'),
      '1800' => t('30 minutes'),
      '3600' => t('60 minutes'),
      '21600' => t('6 hours'),
      '43200' => t('12 hours'),
      '86400' => t('24 hours'),
    ],
    '#description' => t('How long to use cached request values before making a new request.'),
  ];

  $form['gtfs_schedule_feed_id'] = [
    '#type' => 'textfield',
    '#title' => t('Feed ID'),
    '#default_value' => variable_get('gtfs_schedule_feed_id', GTFS_SCHEDULE_FEED_ID),
    '#description' => t('Leave empty for default feed, or add feed ID, or add `latest`.'),
  ];

  $directory = GTFS_SCHEDULE_CACHE_LOCATION;
  $count = 0;
  if (file_exists($directory)) {
    $files = scandir($directory);
    if (count($files) > 2) {
      $form['clear_cache'] = [
        '#type' => 'button',
        '#value' => t('Delete GTFS schedule request cache (count: !count)', ['!count' => (count($files) - 2)]),
        '#name' => 'clear_cache',
      ];

      $form['invalidate_cache'] = [
        '#type' => 'button',
        '#value' => t('Invalidate GTFS schedule request cache (count: !count)', ['!count' => (count($files) - 2)]),
        '#name' => 'invalidate_cache',
      ];
    }
  }

  return system_settings_form($form);
}

/**
 * Submit handler for gtfs_schedule_form.
 */
function gtfs_schedule_settings_form_validate($form, &$form_state) {

  if ($form_state['clicked_button']['#name'] == 'clear_cache') {

    // Clear cache.
    gtfs_schedule_modify_cache('clear_cache');

    // Redirect to avoid saving settings.
    drupal_goto($_GET['q']);
  }

  if ($form_state['clicked_button']['#name'] == 'invalidate_cache') {

    // Invalidate cache.
    gtfs_schedule_modify_cache('invalidate_cache');

    // Redirect to avoid saving settings.
    drupal_goto($_GET['q']);
  }

}

/**
 * Implements hook_flush_caches().
 */
function gtfs_schedule_modify_cache($op) {

  $backdate = strtotime('-1 year');

  $directory = GTFS_SCHEDULE_CACHE_LOCATION;
  if(file_exists($directory)) {
    foreach(scandir($directory) as $file) {

      // Skip directories.
      if(is_dir($file)) {
        continue;
      }

      $path = "{$directory}/{$file}";

      if($op == 'clear_cache') {
        // Remove file.
        unlink($path);
      }
      elseif($op == 'invalidate_cache') {
        // Backdate file by $backdate.
        touch($path, $backdate);
      }
    }
  }

  watchdog(__FUNCTION__, 'GTFS schedule ' . $op);
}


